
#include <iostream>
#include <mutex>
#include "Instantiation_and_registry.h"
#include "ASIO_device.h"
#include "ExchangeBuffer.h"

#define PI 3.14159265

const double sinewave_hz = 220;
const int top = 0x7FFFFFFF;
void sineWave(int hz, float amplitude, int* buffer, int buffer_size, int sampleRate, long samplePos) {
	for (int i = 0; i < buffer_size; i++) {
		float value = amplitude * std::sin(PI * 2 * samplePos / sampleRate * hz);
		buffer[i] = (int)value;
		samplePos++;
	}
}


class Aggregate : public AudioHost {
public:
    ASIODevice dev1;
    ASIODevice dev2;
    ExchangeBuffer* transfer_1_2;
    ExchangeBuffer* transfer_2_1;

    int32_t* temp_buffer[2]{nullptr};
    std::mutex exchange_lock;
    bool exchange_ready = false;

    int call_difference = 0;
    int call_difference_last = 0;
	std::chrono::system_clock::time_point last_pair_change;
    bool first_pair_change = true;

    int max = 0;
    int min = 0;

    const int32_t* SIGNAL_IN;
          int32_t* SIGNAL_OUT;
    const int SIGNAL_IN_CHAN = 0;
    const int SIGNAL_OUT_CHAN = 0;

    Aggregate(int device_index_1, int device_index_2) {
        device_index_1 = 9;
        device_index_2 = 5;

        dev1.attach(device_index_1, this);
        dev1.setChannelStatus(true, SIGNAL_IN_CHAN, true);
        dev2.attach(device_index_2, this);
        dev2.setChannelStatus(false, SIGNAL_OUT_CHAN, true);

        int buffer_size = dev1.get_buffer_size();
        transfer_1_2 = new ExchangeBuffer(buffer_size);
        //transfer_2_1 = new ExchangeBuffer(buffer_size);
        temp_buffer[0] = new int32_t[buffer_size] {0};
        temp_buffer[1] = new int32_t[buffer_size] {0};

        exchange_ready = true;
    }

    void stop() {
        dev1.stop();
        dev2.stop();
    }

    void start() {
        dev1.start();
        dev2.start();
    }
    
    void bufferSwitch(long dbi, bool directProcess, ASIODevice* caller) override {
        // Find which device we are working with
        int behringer = caller->get_device_index() == dev1.get_device_index();
        int motu = !behringer;

        const long buffersize = caller->get_buffer_size();
        const long buffersize_b = buffersize * sizeof(int32_t);

        if (!exchange_ready) {
            caller->output_ready();
            return;
        }

        if (behringer) {
            SIGNAL_IN = dev1.get_input(SIGNAL_IN_CHAN);
            transfer_1_2->writeNext(SIGNAL_IN);
            //SIGNAL_OUT  = dev1.get_output(SIGNAL_OUT_CHAN);
            //transfer_2_1->readNext(this->SIGNAL_OUT);
        } else {
            //SIGNAL_IN   = dev2.get_input(SIGNAL_IN_CHAN);
            //transfer_2_1->writeNext(this->SINGAL_IN);
            SIGNAL_OUT = dev2.get_output(SIGNAL_OUT_CHAN);
            transfer_1_2->readNext(SIGNAL_OUT);
        }
        // Exchange complete
        caller->output_ready();
    }

    long asioMessage(long selector, long value, void* message, double* opt, ASIODevice* caller) override {
		//cout << "#Selector1: " << selector << "   #Value: " << value << "   #Message: " << endl;

		char* message_char = (char*)message;

		// Device asking for supported selectors
		if (selector == kAsioSelectorSupported)
			switch (value) {
			case kAsioEngineVersion:	return 1L;
			case kAsioSupportsTimeInfo: return 1L;
			case kAsioResetRequest:		return 1L;
			case kAsioBufferSizeChange: return 1L;
			case kAsioResyncRequest:	return 0L;
			case kAsioLatenciesChanged: return 0L;
			case kAsioOverload:
                std::cout << "Yes, we support the overload message." << std::endl;
                return 1L;

			default:					return 0;
			}

		switch (selector) {
		case kAsioEngineVersion:	return 2L;
		case kAsioResetRequest:		return 0L;
		case kAsioResyncRequest:	return 0L;
		case kAsioSupportsTimeInfo: return 0L;
		case kAsioOverload:
            std::cout << "Hardware dropout for device " << caller->get_device_index() << std::endl;
            return 1L;
		}

		return 0L;
	}

    void shutdown() {
        dev1.detach();
        dev2.detach();
        delete(temp_buffer[0]);
        delete(temp_buffer[1]);
        //delete(transfer_1_2);
        //delete(transfer_2_1);
    }
private:
    void memsum(int32_t* _Dst, const int32_t* _Src, size_t _Size) {
        for (; _Size > 0; _Size--)
            *_Dst++ += *_Src++;
    }
};

bool choose_device_dialogue(int& device_index_1, int& device_index_2) {
    const int num_devices = ASIO_SDK::get_num_ASIO_drivers();
    std::wstring* names = new std::wstring[num_devices];
    ASIO_SDK::read_driver_name_all(names);

    int index = 0;
    std::wstring* name = &names[0];
    while (index++ - num_devices)
        std::wcout << index << ":" << *name++ << std::endl;
    
    std::cout << "Choose the first audio interface" << std::endl;
	std::string s;
	std::cin >> s;
	device_index_1 = strtol(s.c_str(), nullptr, 0) - 1;
	if (device_index_1 < 0 || device_index_1 >= num_devices) {
		std::cout << "Did not understand the input. Exiting..." << std::endl;
		return false;
	}

    std::cout << "Choose the second audio interface" << std::endl;
	std::cin >> s;
	device_index_2 = strtol(s.c_str(), nullptr, 0) - 1;
	if (device_index_2 < 0 || device_index_2 >= num_devices) {
		std::cout << "Did not understand the input. Exiting..." << std::endl;
		return false;
	}

    return true;
}

int main() {
    int index1=9, index2=5;
    //if (!choose_device_dialogue(index1, index2))
        //return 1;
    
    Aggregate host(index1, index2);
    //host.stop();
    //host.bufferSwitch(0, true, &host.dev1);

    std::cout << "Test now running. Enter anything to stop the test" << std::endl;
	std::string s;
    std::cin >> s;
    std::cout << "Shutting down..." << std::endl;

    host.shutdown();

    return 0;
}