#pragma once

#include "asio_sdk/ginclude.h"
#include "asio_sdk/asio.h"



#define MAX_SIMULTANEOUS_ASIO_DEVICES 32



class ASIO_callback_registry_entry {
public:
	virtual void		bufferSwitch(long doubleBufferIndex, ASIOBool directProcess) = 0;
	virtual void		sampleRateDidChange(ASIOSampleRate sRate) = 0;
	virtual long		asioMessage(long selector, long value, void* message, double* opt) = 0;
	virtual ASIOTime*	bufferSwitchTimeInfo(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) = 0;
};



class ASIO_callback_registry {
public:
	static int  registerEntry(ASIO_callback_registry_entry* entry);
	static void removeEntry  (int index);

	static void getCallbackPointers(int index, ASIOCallbacks* result);


private:
	static ASIO_callback_registry_entry* entries[MAX_SIMULTANEOUS_ASIO_DEVICES];
	static int getAvailableIndex();


#pragma region Static callback definitions

#pragma region BufferSwitch
	static void bufferSwitch_common(int index, long doubleBufferIndex, ASIOBool directProcess) {
		ASIO_callback_registry::entries[index]->bufferSwitch(doubleBufferIndex, directProcess);
	}
	static void bufferSwitch_0 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 0, doubleBufferIndex, directProcess); }
	static void bufferSwitch_1 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 1, doubleBufferIndex, directProcess); }
	static void bufferSwitch_2 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 2, doubleBufferIndex, directProcess); }
	static void bufferSwitch_3 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 3, doubleBufferIndex, directProcess); }
	static void bufferSwitch_4 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 4, doubleBufferIndex, directProcess); }
	static void bufferSwitch_5 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 5, doubleBufferIndex, directProcess); }
	static void bufferSwitch_6 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 6, doubleBufferIndex, directProcess); }
	static void bufferSwitch_7 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 7, doubleBufferIndex, directProcess); }
	static void bufferSwitch_8 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 8, doubleBufferIndex, directProcess); }
	static void bufferSwitch_9 (long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common( 9, doubleBufferIndex, directProcess); }
	static void bufferSwitch_10(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(10, doubleBufferIndex, directProcess); }
	static void bufferSwitch_11(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(11, doubleBufferIndex, directProcess); }
	static void bufferSwitch_12(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(12, doubleBufferIndex, directProcess); }
	static void bufferSwitch_13(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(13, doubleBufferIndex, directProcess); }
	static void bufferSwitch_14(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(14, doubleBufferIndex, directProcess); }
	static void bufferSwitch_15(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(15, doubleBufferIndex, directProcess); }
	static void bufferSwitch_16(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(16, doubleBufferIndex, directProcess); }
	static void bufferSwitch_17(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(17, doubleBufferIndex, directProcess); }
	static void bufferSwitch_18(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(18, doubleBufferIndex, directProcess); }
	static void bufferSwitch_19(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(19, doubleBufferIndex, directProcess); }
	static void bufferSwitch_20(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(20, doubleBufferIndex, directProcess); }
	static void bufferSwitch_21(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(21, doubleBufferIndex, directProcess); }
	static void bufferSwitch_22(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(22, doubleBufferIndex, directProcess); }
	static void bufferSwitch_23(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(23, doubleBufferIndex, directProcess); }
	static void bufferSwitch_24(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(24, doubleBufferIndex, directProcess); }
	static void bufferSwitch_25(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(25, doubleBufferIndex, directProcess); }
	static void bufferSwitch_26(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(26, doubleBufferIndex, directProcess); }
	static void bufferSwitch_27(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(27, doubleBufferIndex, directProcess); }
	static void bufferSwitch_28(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(28, doubleBufferIndex, directProcess); }
	static void bufferSwitch_29(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(29, doubleBufferIndex, directProcess); }
	static void bufferSwitch_30(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(30, doubleBufferIndex, directProcess); }
	static void bufferSwitch_31(long doubleBufferIndex, ASIOBool directProcess) { bufferSwitch_common(31, doubleBufferIndex, directProcess); }
#pragma endregion

#pragma region BufferSwitchTimeInfo
	static ASIOTime* bufferSwitchTimeInfo_common(int index, ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) {
		return ASIO_callback_registry::entries[index]->bufferSwitchTimeInfo(params, doubleBufferIndex, directProcess);
	}
	static ASIOTime* bufferSwitchTimeInfo_0 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 0, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_1 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 1, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_2 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 2, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_3 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 3, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_4 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 4, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_5 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 5, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_6 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 6, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_7 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 7, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_8 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 8, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_9 (ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common( 9, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_10(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(10, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_11(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(11, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_12(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(12, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_13(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(13, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_14(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(14, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_15(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(15, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_16(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(16, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_17(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(17, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_18(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(18, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_19(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(19, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_20(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(20, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_21(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(21, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_22(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(22, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_23(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(23, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_24(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(24, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_25(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(25, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_26(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(26, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_27(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(27, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_28(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(28, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_29(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(29, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_30(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(30, params, doubleBufferIndex, directProcess); }
	static ASIOTime* bufferSwitchTimeInfo_31(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return bufferSwitchTimeInfo_common(31, params, doubleBufferIndex, directProcess); }
#pragma endregion

#pragma region AsioMessage
	static long asioMessage_common(int index, long selector, long value, void* message, double* opt) {
		return ASIO_callback_registry::entries[index]->asioMessage(selector, value, message, opt);
	}
	static long asioMessage_0 (long selector, long value, void* message, double* opt) { return asioMessage_common( 0, selector, value, message, opt); }
	static long asioMessage_1 (long selector, long value, void* message, double* opt) { return asioMessage_common( 1, selector, value, message, opt); }
	static long asioMessage_2 (long selector, long value, void* message, double* opt) { return asioMessage_common( 2, selector, value, message, opt); }
	static long asioMessage_3 (long selector, long value, void* message, double* opt) { return asioMessage_common( 3, selector, value, message, opt); }
	static long asioMessage_4 (long selector, long value, void* message, double* opt) { return asioMessage_common( 4, selector, value, message, opt); }
	static long asioMessage_5 (long selector, long value, void* message, double* opt) { return asioMessage_common( 5, selector, value, message, opt); }
	static long asioMessage_6 (long selector, long value, void* message, double* opt) { return asioMessage_common( 6, selector, value, message, opt); }
	static long asioMessage_7 (long selector, long value, void* message, double* opt) { return asioMessage_common( 7, selector, value, message, opt); }
	static long asioMessage_8 (long selector, long value, void* message, double* opt) { return asioMessage_common( 8, selector, value, message, opt); }
	static long asioMessage_9 (long selector, long value, void* message, double* opt) { return asioMessage_common( 9, selector, value, message, opt); }
	static long asioMessage_10(long selector, long value, void* message, double* opt) { return asioMessage_common(10, selector, value, message, opt); }
	static long asioMessage_11(long selector, long value, void* message, double* opt) { return asioMessage_common(11, selector, value, message, opt); }
	static long asioMessage_12(long selector, long value, void* message, double* opt) { return asioMessage_common(12, selector, value, message, opt); }
	static long asioMessage_13(long selector, long value, void* message, double* opt) { return asioMessage_common(13, selector, value, message, opt); }
	static long asioMessage_14(long selector, long value, void* message, double* opt) { return asioMessage_common(14, selector, value, message, opt); }
	static long asioMessage_15(long selector, long value, void* message, double* opt) { return asioMessage_common(15, selector, value, message, opt); }
	static long asioMessage_16(long selector, long value, void* message, double* opt) { return asioMessage_common(16, selector, value, message, opt); }
	static long asioMessage_17(long selector, long value, void* message, double* opt) { return asioMessage_common(17, selector, value, message, opt); }
	static long asioMessage_18(long selector, long value, void* message, double* opt) { return asioMessage_common(18, selector, value, message, opt); }
	static long asioMessage_19(long selector, long value, void* message, double* opt) { return asioMessage_common(19, selector, value, message, opt); }
	static long asioMessage_20(long selector, long value, void* message, double* opt) { return asioMessage_common(20, selector, value, message, opt); }
	static long asioMessage_21(long selector, long value, void* message, double* opt) { return asioMessage_common(21, selector, value, message, opt); }
	static long asioMessage_22(long selector, long value, void* message, double* opt) { return asioMessage_common(22, selector, value, message, opt); }
	static long asioMessage_23(long selector, long value, void* message, double* opt) { return asioMessage_common(23, selector, value, message, opt); }
	static long asioMessage_24(long selector, long value, void* message, double* opt) { return asioMessage_common(24, selector, value, message, opt); }
	static long asioMessage_25(long selector, long value, void* message, double* opt) { return asioMessage_common(25, selector, value, message, opt); }
	static long asioMessage_26(long selector, long value, void* message, double* opt) { return asioMessage_common(26, selector, value, message, opt); }
	static long asioMessage_27(long selector, long value, void* message, double* opt) { return asioMessage_common(27, selector, value, message, opt); }
	static long asioMessage_28(long selector, long value, void* message, double* opt) { return asioMessage_common(28, selector, value, message, opt); }
	static long asioMessage_29(long selector, long value, void* message, double* opt) { return asioMessage_common(29, selector, value, message, opt); }
	static long asioMessage_30(long selector, long value, void* message, double* opt) { return asioMessage_common(30, selector, value, message, opt); }
	static long asioMessage_31(long selector, long value, void* message, double* opt) { return asioMessage_common(31, selector, value, message, opt); }
#pragma endregion

#pragma region SampleRateDidChange
	static void sampleRateDidChange_common(int index, ASIOSampleRate sRate) {
		ASIO_callback_registry::entries[index]->sampleRateDidChange(sRate);
	}
	static void sampleRateDidChange_0 (ASIOSampleRate sRate) { sampleRateDidChange_common( 0, sRate); }
	static void sampleRateDidChange_1 (ASIOSampleRate sRate) { sampleRateDidChange_common( 1, sRate); }
	static void sampleRateDidChange_2 (ASIOSampleRate sRate) { sampleRateDidChange_common( 2, sRate); }
	static void sampleRateDidChange_3 (ASIOSampleRate sRate) { sampleRateDidChange_common( 3, sRate); }
	static void sampleRateDidChange_4 (ASIOSampleRate sRate) { sampleRateDidChange_common( 4, sRate); }
	static void sampleRateDidChange_5 (ASIOSampleRate sRate) { sampleRateDidChange_common( 5, sRate); }
	static void sampleRateDidChange_6 (ASIOSampleRate sRate) { sampleRateDidChange_common( 6, sRate); }
	static void sampleRateDidChange_7 (ASIOSampleRate sRate) { sampleRateDidChange_common( 7, sRate); }
	static void sampleRateDidChange_8 (ASIOSampleRate sRate) { sampleRateDidChange_common( 8, sRate); }
	static void sampleRateDidChange_9 (ASIOSampleRate sRate) { sampleRateDidChange_common( 9, sRate); }
	static void sampleRateDidChange_10(ASIOSampleRate sRate) { sampleRateDidChange_common(10, sRate); }
	static void sampleRateDidChange_11(ASIOSampleRate sRate) { sampleRateDidChange_common(11, sRate); }
	static void sampleRateDidChange_12(ASIOSampleRate sRate) { sampleRateDidChange_common(12, sRate); }
	static void sampleRateDidChange_13(ASIOSampleRate sRate) { sampleRateDidChange_common(13, sRate); }
	static void sampleRateDidChange_14(ASIOSampleRate sRate) { sampleRateDidChange_common(14, sRate); }
	static void sampleRateDidChange_15(ASIOSampleRate sRate) { sampleRateDidChange_common(15, sRate); }
	static void sampleRateDidChange_16(ASIOSampleRate sRate) { sampleRateDidChange_common(16, sRate); }
	static void sampleRateDidChange_17(ASIOSampleRate sRate) { sampleRateDidChange_common(17, sRate); }
	static void sampleRateDidChange_18(ASIOSampleRate sRate) { sampleRateDidChange_common(18, sRate); }
	static void sampleRateDidChange_19(ASIOSampleRate sRate) { sampleRateDidChange_common(19, sRate); }
	static void sampleRateDidChange_20(ASIOSampleRate sRate) { sampleRateDidChange_common(20, sRate); }
	static void sampleRateDidChange_21(ASIOSampleRate sRate) { sampleRateDidChange_common(21, sRate); }
	static void sampleRateDidChange_22(ASIOSampleRate sRate) { sampleRateDidChange_common(22, sRate); }
	static void sampleRateDidChange_23(ASIOSampleRate sRate) { sampleRateDidChange_common(23, sRate); }
	static void sampleRateDidChange_24(ASIOSampleRate sRate) { sampleRateDidChange_common(24, sRate); }
	static void sampleRateDidChange_25(ASIOSampleRate sRate) { sampleRateDidChange_common(25, sRate); }
	static void sampleRateDidChange_26(ASIOSampleRate sRate) { sampleRateDidChange_common(26, sRate); }
	static void sampleRateDidChange_27(ASIOSampleRate sRate) { sampleRateDidChange_common(27, sRate); }
	static void sampleRateDidChange_28(ASIOSampleRate sRate) { sampleRateDidChange_common(28, sRate); }
	static void sampleRateDidChange_29(ASIOSampleRate sRate) { sampleRateDidChange_common(29, sRate); }
	static void sampleRateDidChange_30(ASIOSampleRate sRate) { sampleRateDidChange_common(30, sRate); }
	static void sampleRateDidChange_31(ASIOSampleRate sRate) { sampleRateDidChange_common(31, sRate); }
#pragma endregion

#pragma endregion
};