#pragma once

#include <stdint.h>

/// @brief This class is the general interface for all classes that provide a solution for the click/pop that arises
/// when transferring audio data from one audio interface to another.
/// Any process that wishes to use this 
class CutMasker {
public:
    virtual ~CutMasker() {};

    /// @brief Masks the pop/click in an audio stream caused by unsychronised audio streams.
    /// The caller will call this fuction every 
    /// @param samples_jumped The number of samples that the provider stream either jumps ahead - or falls behind - of the consumer stream.
    /// A positive value says the provider jumps ahead of the consumer by 'samples_jumped' number of samples.
    /// A negative value says the provider falls behind the consumer by 'samples_jumped' number of samples.
    /// @param input
    /// @param output 
    /// @param buffersize 
    virtual void writeNext(int16_t samples_jumped, const int32_t* &source,      const uint16_t &buffersize) = 0;
    virtual void readNext (int16_t samples_jumped,       int32_t* &destination, const uint16_t &buffersize) = 0;
};