
#include <stdint.h>
#include <mutex>

#include "SampleJumper2.h"

class ExchangeBuffer {
public:

    ExchangeBuffer();
    ExchangeBuffer(const size_t&  buffer_size);

    void set_buffer_size(const size_t& buffersize);
    void readNext(int32_t* destination);
    void writeNext(const int32_t* &data);

    const size_t& buffersize() const { return _buffersize; };

private:
    static const int NUM_BUFFERS = 5;
    static const int8_t THRESHOLD_DISTANCE = 3;

    size_t _buffersize;
    size_t _buffersize_in_bytes;
    std::mutex transfer_lock;

    int32_t* buffers[NUM_BUFFERS];

    int8_t readpos;
    int8_t writepos;
    int8_t distance;
    SampleJumper* masker;

     /// Indicates wether both devices has started up completely.
     /// According to the doc, devices don't usually start up before after the third bufferswitch-call.
     /// And because of the synchronisation-functionality requiring both device to execute consecutively,
     /// we make sure to wait for both devices to start up (aka. perform 3 calls) before processing. 
    bool ignition;
    uint8_t  read_dev_ignition_runs;
    uint8_t write_dev_ignition_runs;
    static const uint8_t call_required_for_startup = 5;

    void  readpos_inc() {  readpos == NUM_BUFFERS - 1 ?  readpos = 0 :  readpos++; }
    void writepos_inc() { writepos == NUM_BUFFERS - 1 ? writepos = 0 : writepos++; }
};