#pragma once

#include <string>
#include <windows.h>

#include "ASIO_driver_interface.h"

#define ASIODRV_DESC		L"description"
#define INPROC_SERVER		L"InprocServer32"
#define ASIO_PATH			L"software\\asio"
#define COM_CLSID			L"clsid"

#define MAX_REG_KEY_LEN		255
#define MAX_DRV_NAME_LEN	128
#define MAX_CLSID_LEN		512


/// <summary>
/// <para> This class is an information-hub. It contains static functions for reading information (from the registry) about installed drivers.															</para>
/// <para> This includes number of installed drivers, their names and CLSIDs.																															</para>
/// <para> And additionally two functions for creating an instance of an audio interface from it's CLSID.																								</para>
///	<para> 																																																</para>
/// <para> Note the abundance of simmilar function names and overloading. You can sort them in three groups: driver names, driver CLSID, and instantiation.												</para>
/// <para> (... and the function for finding the number of installed drivers, but that's only one function, and it's prescence is kind of expected anyways, so it does not deserve it's own group)		</para>
///	<para> 																																																</para>
/// <para> The important thing to note is that these functions require an index.																														</para>
/// <para> This index has to be more than or equal to 0, and less than the total number of installed drivers (found by calling 'get_num_ASIO_drivers()').												</para>
/// <para> Which indices points to which drivers? That is decided by how the drivers are listed in the registry.																						</para>
/// <para> Still, that is of little worry to you, because this index-system is consistent across all functions in this class.																			</para>
/// </summary>
class ASIO_SDK {
public:
	// Read number of installed drivers.
	static int get_num_ASIO_drivers();

	// Reading driver name from registry.
	static std::wstring read_driver_name(int index);
	static void    read_driver_name(int index, std::wstring* result);
	static void    read_driver_names(int* indices, size_t size, std::wstring* result);
	static int     read_driver_name_all(std::wstring* result);

	// Reading CLSID from registry.
	static CLSID   read_driver_CLSID(int index);
	static void    read_driver_CLSID(int index, CLSID*   result);
	static void    read_driver_CLSID(int index, std::wstring* result);
	static std::wstring read_driver_CLSID_str(int index);

	static int read_driver_CLSID_all(CLSID*   result);
	static int read_driver_CLSID_all(std::wstring* result);

	// Reading everything we can find!
	static int read_ASIO_driver_names_and_CLSID(std::wstring* driver_names, CLSID*   driver_ids);
	static int read_ASIO_driver_names_and_CLSID(std::wstring* driver_names, std::wstring* driver_ids);

	// Instantiation; create an instance of IASIO that can control the audio interface.
	static HRESULT createInstance(CLSID driver_id, IASIO** instance);
	static  IASIO* createInstance(CLSID driver_id);
	static HRESULT createInstance(int index, IASIO** instance);
	static  IASIO* createInstance(int index);

	// Checking for device status
	/*
	static int  get_num_connected_devices();
	static bool is_device_connected(int device_index);
	static bool is_device_connected(IASIO* device_index);
	static int  read_all_connected_devices(int**  device_indices);
	*/
private:
	static int get_all_ASIO_keys(std::wstring* keys);
	static HKEY open_regKey_to_ASIO_drivers();
};