#pragma once

#include <stdint.h>
#include "CutMasker.h"
#include "exchange/RingBuffer.h"
#include "TimeCompressor.h"

#define SAMPLE int32_t
#define INDEX int

class SampleJumper : public CutMasker {
public:
    /// @brief Creates a new instance of SampleJumper. 
    /// @param hardware_buffersize The size of the buffer shared between this software and the devices' driver. It represents the largest amount of samples expected to be sent to both writeNext and readNext.
    /// This value decides the size of the internal buffer, and directly impact the resulting delay. 
    SampleJumper(const int16_t& hardware_buffersize);
    ~SampleJumper();

    void writeNext(int16_t samples_jumped, const int32_t* &source,      const uint16_t &count);
    void readNext (int16_t samples_jumped,       int32_t* &destination, const uint16_t &count);

    void set_threshold_volume(const float  decibels_0DBFS);
    void set_max_sample_jump(const int32_t sample_val_threshold);

private:
    RingBuffer buffer;
    TimeCompressor tcomp;
    SAMPLE* input_buffer;

    SAMPLE* temp_buffer_1;
    SAMPLE* temp_buffer_2;
    
    INDEX buffersize;
    INDEX hardware_buffersize;

    INDEX threshold_min;
    INDEX threshold_max;

    SAMPLE samples_offset;
    SAMPLE sample_diff_threshold;
    //INDEX sample_change_interval;
    bool faster_consumer; // True means that the audio consumer has a faster clock than the provider, and that this SampleJumper has to delay the signal by one buffer.

    static const SAMPLE sample_0dBFS = 2147483647;

    INDEX try_compress(const INDEX& num_samples);
    INDEX try_lengthen(const INDEX& num_samples);
};

#undef SAMPLE
#undef INDEX