#pragma once

#include <mutex>

#define RB_DATA int32_t
#define INDEX int

class RingBuffer {
public:
    const INDEX buffer_size;

    RingBuffer(const INDEX& buffer_size);
    ~RingBuffer();

    void read_next(RB_DATA* destination, const INDEX& amount);
    void write_next(const RB_DATA* source, const INDEX& amount);
    
    void ignore_xruns(const bool& ignore_xruns);
    const bool& ignoring_xruns() const;
    
    void  read_cursor_inc(const INDEX& increment);
    void write_cursor_inc(const INDEX& increment);
    void  read_cursor_dec(const INDEX& decrement);
    void write_cursor_dec(const INDEX& decrement);
    void skip_all();
    
    /// @brief Copies the content of this ringbuffer to the submitted buffer without moving any of the cursors.
    /// @param destination 
    /// @return the numer of elements written to the destination buffer.
    INDEX get_data(RB_DATA* destination) const;
    void  set_data(const RB_DATA* source, INDEX size);

    RB_DATA* get_buffer() const;
    const INDEX& get_data_amount() const;
    const INDEX& get_read_cursor() const;
    const INDEX& get_write_cursor() const;
    
protected:
    RB_DATA* buffer;
    const uint8_t INDEX_BYTE_SIZE;
    bool _ignore_xruns;

    INDEX read_cursor;
    INDEX write_cursor;
    INDEX cursor_distance;
    bool roundtrip;

    std::mutex lock;
};

#undef RB_DATA
#undef INDEX