
#include <iostream>

#include "ExchangeBuffer.h"
#include "SampleJumper2.h"

ExchangeBuffer::ExchangeBuffer() : ExchangeBuffer(256) {}
ExchangeBuffer::ExchangeBuffer(const size_t &buffersize) {
    this->readpos = 0;
    this->writepos = 0;
    this->distance = 0;
    this->ignition = false;
    this->read_dev_ignition_runs = 0;
    this->write_dev_ignition_runs = 0;
    set_buffer_size(buffersize);
}

void ExchangeBuffer::set_buffer_size(const size_t& buffersize) {
    transfer_lock.lock();
    this->_buffersize = buffersize;
    this->_buffersize_in_bytes = buffersize * sizeof(int32_t);
    for (int i = 0; i < NUM_BUFFERS; i++)
        buffers[i] = new int32_t[buffersize];
    this->masker = new SampleJumper(buffersize);
    transfer_lock.unlock();
}

void ExchangeBuffer::readNext(int32_t* destination) {
    if (!ignition) {
        if (read_dev_ignition_runs < call_required_for_startup)
            read_dev_ignition_runs++;
        return;
    }

    std::size_t samples_underflown = 0;

    transfer_lock.lock();
    if (distance > 0) {
        // readpos_inc();
        distance--;
    } else {
        // Coming here means a buffer has been skipped!
        // The reader has caught up with the writer, and has to wait for the writer to write more data.
        // It waits by continously reading the same data over and over again.
        // Write som code here to create a reaction to this event.
        samples_underflown = -_buffersize;
    }
    
    // We do not need the reader to wait any further. If an underrun has happenend by now, there is nothing more we can do here to fix it.
    // That is up to the masker.
    // Better to let the writer run along immediately.
    masker->readNext(samples_underflown, destination, _buffersize);
    transfer_lock.unlock();
    //memcpy(destination, buffers[readpos], _buffersize_in_bytes);
}

void ExchangeBuffer::writeNext(const int32_t*& data) {
    if (!ignition) {
        if (write_dev_ignition_runs < call_required_for_startup)
            write_dev_ignition_runs++;
        
        // Ready to start? Only the writer can ignite the exchange; it must run first.
        if (read_dev_ignition_runs >= call_required_for_startup && write_dev_ignition_runs >= call_required_for_startup) {
            ignition = true; // IGNITION!
            std::cout << "IGNITION!" << std::endl;
        }
        else
            return;
    }

    std::size_t samples_overflown = 0;

    transfer_lock.lock();
    if (distance < THRESHOLD_DISTANCE) {
        // writepos_inc();
        distance++;
    } else {
        // Coming here means a buffer has been skipped!
        // The writer has written more than the reader can read, and has to wait for the reader before being allowed to move one buffer forward.
        // It waits by continously writing to the same memory location over and over again.
        // Write som code here to create a reaction to this event.
        samples_overflown += _buffersize;
    }

    // We have to make the reader wait for the writer to be finished writing all the data, 
    // in case the reader is about to read the same memory location as the writer is writing to.
    masker->writeNext(samples_overflown, data, _buffersize);
    //memcpy(buffers[writepos], data, _buffersize_in_bytes); 
    transfer_lock.unlock();
}