
#if defined(_WIN32) || defined(_WIN64)
// This is the a Windows specific implementation

#include <stdexcept>
#include <vector>
#include "asio_sdk/ginclude.h"
#include "Instantiation_and_registry.h"

std::wstring ASIO_SDK::read_driver_name(int index) {
	std::wstring result_s;
	read_driver_name(index, &result_s);
	return result_s;
}

std::wstring ASIO_SDK::read_driver_CLSID_str(int index) {
	std::wstring result_s;
	read_driver_CLSID(index, &result_s);
	return result_s;
}

CLSID ASIO_SDK::read_driver_CLSID(int index) {
	CLSID result_s;
	read_driver_CLSID(index, &result_s);
	return result_s;
}

void ASIO_SDK::read_driver_name(int index, std::wstring* result) {
	int num_drivers = get_num_ASIO_drivers();

	if (index < 0) {
		throw std::invalid_argument("Negative index value.");
		return;
	}

	if (index >= num_drivers) {
		throw std::invalid_argument("Index is too high.");
		return;
	}

	if (result == nullptr) {
		throw std::invalid_argument("'Result' parameter cannot be null.");
		return;
	}

	HKEY registry_key = open_regKey_to_ASIO_drivers();

	std::vector<std::wstring> sub_keys(num_drivers);
	get_all_ASIO_keys(&sub_keys[0]);

	DWORD	name_type = 0;
	DWORD	name_len = MAX_DRV_NAME_LEN;
	wchar_t	name_buffer[MAX_DRV_NAME_LEN];
	int response = RegGetValueW(registry_key, sub_keys[index].c_str(), ASIODRV_DESC, RRF_RT_REG_SZ, &name_type, name_buffer, &name_len);
	
	*result = std::wstring(name_buffer);

	RegCloseKey(registry_key);
}

void ASIO_SDK::read_driver_names(int* indices, size_t size, std::wstring* result) {
	while(size--)
		read_driver_name(*indices++, result++);
}

void ASIO_SDK::read_driver_CLSID(int index, std::wstring* result) {
	int num_drivers = get_num_ASIO_drivers();

	if (index < 0) {
		throw std::invalid_argument("Negative index value.");
		return;
	}

	if (index >= num_drivers) {
		throw std::invalid_argument("Index is too high.");
		return;
	}

	if (result == nullptr) {
		throw std::invalid_argument("'Result' parameter cannot be null.");
		return;
	}


	HKEY registry_key = open_regKey_to_ASIO_drivers();

	std::vector<std::wstring> sub_keys(num_drivers);
	get_all_ASIO_keys(&sub_keys[0]);

	DWORD	id_type = 0;
	DWORD	id_len = MAX_CLSID_LEN;
	wchar_t	id_buffer[MAX_CLSID_LEN];
	int respon = RegGetValueW(registry_key, sub_keys[index].c_str(), COM_CLSID, RRF_RT_REG_SZ, &id_type, id_buffer, &id_len);

	*result = std::wstring(id_buffer);

	RegCloseKey(registry_key);
}

void ASIO_SDK::read_driver_CLSID(int index, CLSID* result) {
	std::wstring result_s;
	ASIO_SDK::read_driver_CLSID(index, &result_s);

	CLSIDFromString(result_s.c_str(), result);
}

int ASIO_SDK::read_driver_name_all(std::wstring* driver_names) {
	int num_drivers = get_num_ASIO_drivers();

	std::vector<std::wstring> subkeys(num_drivers);
	get_all_ASIO_keys(&subkeys[0]);

	HKEY registry_key = open_regKey_to_ASIO_drivers();

	for (int i = 0; i < num_drivers; i++) {
		DWORD	name_type = 0;
		DWORD	name_len = MAX_DRV_NAME_LEN;
		wchar_t	name_buffer[MAX_DRV_NAME_LEN];
		int respon = RegGetValueW(registry_key, subkeys[i].c_str(), ASIODRV_DESC, RRF_RT_REG_SZ, &name_type, name_buffer, &name_len);

		driver_names[i] = std::wstring(name_buffer);
	}

	RegCloseKey(registry_key);
	return num_drivers;
}

int ASIO_SDK::read_driver_CLSID_all(std::wstring* result) {
	int num_drivers = get_num_ASIO_drivers();

	std::vector<std::wstring> subkeys(num_drivers);
	get_all_ASIO_keys(&subkeys[0]);

	HKEY registry_key = open_regKey_to_ASIO_drivers();

	for (int i = 0; i < num_drivers; i++) {
		DWORD	id_type = 0;
		DWORD	id_len = MAX_DRV_NAME_LEN;
		wchar_t	id_buffer[MAX_DRV_NAME_LEN];
		int respon = RegGetValueW(registry_key, subkeys[i].c_str(), COM_CLSID, RRF_RT_REG_SZ, &id_type, id_buffer, &id_len);

		result[i] = std::wstring(id_buffer);
	}

	RegCloseKey(registry_key);
	return num_drivers;
}

int ASIO_SDK::read_driver_CLSID_all(CLSID* result) {
	int num_drivers = get_num_ASIO_drivers();
	std::vector<std::wstring> result_s(num_drivers);

	for (int i = 0; i < num_drivers; i++) {
		CLSIDFromString(result_s[i].c_str(), &result[i]);
	}

	return 0;
}

int ASIO_SDK::read_ASIO_driver_names_and_CLSID(std::wstring* driver_names, std::wstring* driver_ids) {
	HKEY registry_key = open_regKey_to_ASIO_drivers();
	
	int num_drivers = get_num_ASIO_drivers();

	std::vector<std::wstring> driver_keys(num_drivers);
	get_all_ASIO_keys(&driver_keys[0]);

	// Enter each subkey, and extract name and clsid
	for (int i = 0; i < num_drivers; i++) {

		DWORD	name_type = 0;
		DWORD	name_len = MAX_DRV_NAME_LEN;
		wchar_t	name_buffer[MAX_DRV_NAME_LEN];

		DWORD	id_type = 0;
		DWORD	id_len = MAX_CLSID_LEN;
		wchar_t	id_buffer[MAX_CLSID_LEN];


		int respon1 = RegGetValueW(registry_key, driver_keys[i].c_str(), ASIODRV_DESC, RRF_RT_REG_SZ, &name_type, name_buffer, &name_len);
		int respon2 = RegGetValueW(registry_key, driver_keys[i].c_str(), COM_CLSID, RRF_RT_REG_SZ, &id_type, id_buffer, &id_len);

		if (respon1 != ERROR_SUCCESS || respon2 != ERROR_SUCCESS)
			break; // This really would surprise me!

		driver_names[i] = std::wstring(name_buffer);
		driver_ids[i] = std::wstring(id_buffer);
	}

	RegCloseKey(registry_key);
	return num_drivers;
}

int ASIO_SDK::read_ASIO_driver_names_and_CLSID(std::wstring* driver_names, CLSID* driver_ids) {
	std::vector<std::wstring> id_string(get_num_ASIO_drivers());

	int num_drivers = read_ASIO_driver_names_and_CLSID(driver_names, &id_string[0]);

	for (int i = 0; i < num_drivers; i++) {
		CLSID id;
		HRESULT result = CLSIDFromString(id_string[i].c_str(), &id);
		driver_ids[i] = id;
	}

	return num_drivers;
}

HRESULT ASIO_SDK::createInstance(CLSID driver_id, IASIO** instance) {
	HRESULT error = S_OK;
	error = CoCreateInstance(driver_id, NULL, CLSCTX_INPROC_SERVER, driver_id, (void**)instance);

	if (error == 0x800401F0) { // If that value is returned by 'CoCreateInstance', it means that CoInitialize has not been called yet.
		CoInitialize(0); // But no worries! I'll fix that for you :D
		error = CoCreateInstance(driver_id, NULL, CLSCTX_INPROC_SERVER, driver_id, (void**)instance);
	}

	return error;
}

IASIO* ASIO_SDK::createInstance(CLSID driver_id) {
	IASIO* new_instance;
	ASIO_SDK::createInstance(driver_id, &new_instance);
	return new_instance;
}

HRESULT ASIO_SDK::createInstance(int index, IASIO** instance) {
	CLSID id;
	ASIO_SDK::read_driver_CLSID(index, &id);
	return ASIO_SDK::createInstance(id, instance);
}

IASIO* ASIO_SDK::createInstance(int index) {
	IASIO* new_instance;
	ASIO_SDK::createInstance(index, &new_instance);
	return new_instance;
}

/*
int ASIO_SDK::get_num_connected_devices() {
	const int num_devices = get_num_ASIO_drivers();
	int num_connected_devices = 0;
	
	for (int i = 0; i < num_devices; i++)
		if (is_device_connected(i))
			num_connected_devices++;

	return num_connected_devices;
}

bool ASIO_SDK::is_device_connected(int device_index) {
	IASIO* device;
	createInstance(device_index, &device);
	device->AddRef();
	DWORD processID = GetCurrentProcessId();
	ASIOError aerr = device->init((void*)&processID);

	bool connected = is_device_connected(device);

	device->Release();

	return connected;
}

bool ASIO_SDK::is_device_connected(IASIO* device) {
	ASIOSampleRate sr;
	
	char* s = new char[32];
	device->getDriverName(s);

	long min, max, pref, gran;
	ASIOError buff_err = device->getBufferSize(&min, &max, &pref, &gran);
	if (buff_err != ASE_OK)
		return false;
	
	ASIOError err = device->createBuffers(new ASIOBufferInfo{ false, 0 }, 1, pref, );
	if (err == ASE_OK)
		device->disposeBuffers();
	return err == ASE_OK;
}

int ASIO_SDK::read_all_connected_devices(int** device_indices) {
	const int num_devices = get_num_ASIO_drivers();
	std::vector<int> connected_devices;

	for (int i = 0; i < num_devices; i++)
		if (is_device_connected(i))
			connected_devices.push_back(i);
	
	*device_indices = new int[connected_devices.size()];

	for (int i = 0; i < connected_devices.size(); i++)
		*device_indices[i] = connected_devices[i];
	return connected_devices.size();
}
*/

int ASIO_SDK::get_num_ASIO_drivers() {
	HKEY registry_key = open_regKey_to_ASIO_drivers();

	wchar_t buffer[1];
	DWORD length = 1;

	int num_drivers = 0;

	while (true) {
		long status = RegEnumKeyExW(registry_key, num_drivers, buffer, &length, NULL, NULL, NULL, NULL);

		if (status == ERROR_NO_MORE_ITEMS)
			break;
		num_drivers++;
	}

	RegCloseKey(registry_key);
	return num_drivers;
}

int ASIO_SDK::get_all_ASIO_keys(std::wstring* keys) {
	int num_keys = get_num_ASIO_drivers();

	HKEY registry_key = open_regKey_to_ASIO_drivers();

	for (int i = 0; i < num_keys; i++) {
		wchar_t name_buffer[MAX_REG_KEY_LEN];
		DWORD	buffer_size = sizeof(name_buffer) / 2;

		long status = RegEnumKeyExW(registry_key, i, name_buffer, &buffer_size, NULL, NULL, NULL, NULL);
		keys[i] = std::wstring(name_buffer);
	}

	RegCloseKey(registry_key);
	return num_keys;
}

HKEY ASIO_SDK::open_regKey_to_ASIO_drivers() {
	HKEY registry_key_to_ASIO_drivers;

	long err = RegOpenKeyExW(HKEY_LOCAL_MACHINE, ASIO_PATH, 0, KEY_READ, &registry_key_to_ASIO_drivers);
	if (err != ERROR_SUCCESS)
		return nullptr;

	return registry_key_to_ASIO_drivers;
}

#endif