
#include <iostream>
#include <exception>
#include "SampleJumper2.h"

#define SAMPLE int32_t
#define INDEX int

SampleJumper::SampleJumper(const int16_t& hardware_buffersize) : buffer(hardware_buffersize * 6), tcomp(hardware_buffersize * 6) {
    this->hardware_buffersize = hardware_buffersize;
    this->buffersize = hardware_buffersize * 6;
    this->input_buffer = new SAMPLE[buffersize] {0};

    tcomp.set_value_threshold(1 << 15);
    tcomp.set_action_interval(64);

    this->temp_buffer_1 = new int32_t[this->buffersize] {0};
    this->temp_buffer_2 = new int32_t[this->buffersize] {0};

    this->threshold_min = hardware_buffersize * 2;
    this->threshold_max = hardware_buffersize * 3;

    this->samples_offset = 0;
    
    this->set_max_sample_jump(1 << 14);
}

SampleJumper::~SampleJumper() {
    delete(this->temp_buffer_1);
    delete(this->temp_buffer_2);
}


void SampleJumper::writeNext(int16_t samples_jumped, const int32_t* &input, const uint16_t& samplecount) {
    INDEX samples_to_fix = buffer.get_data_amount() - threshold_max;

    int fixed = 0;
    if (samples_to_fix > 0) {
        tcomp.set_target(samples_to_fix);
        memcpy(input_buffer, input, samplecount * sizeof(SAMPLE));
        fixed = tcomp.compressNext(input_buffer, samplecount);
        //std::cout << "Try comp" << std::endl;
    }

    if (fixed)
        std::cout << "Compress " << fixed << std::endl;
    
    if (buffer.buffer_size >= buffer.get_data_amount() + samplecount - fixed) {
        if (fixed)
            buffer.write_next(input_buffer, samplecount - fixed);
        else
            buffer.write_next(input, samplecount);
    }
    else
        std::cout << "Overrun" << std::endl;
}

void SampleJumper::readNext(int16_t samples_jumped, int32_t* &output, const uint16_t& samplecount) {
    if (samples_jumped) {
        samples_offset = samples_jumped;
        std::cout << samples_jumped << std::endl;
    }

    INDEX samples_to_fix = threshold_min - buffer.get_data_amount();
    samples_to_fix -= samples_offset;

    int fixed = 0;
    if (samples_to_fix > 0)
        fixed = try_lengthen(samples_to_fix);
    if (fixed)
        std::cout << "Strekk " << fixed << std::endl;
    if (fixed && samples_offset)
        samples_offset += fixed;
    
    if (buffer.get_data_amount() >= samplecount)
        buffer.read_next(output, samplecount);
    else {
        samples_offset += samplecount;
        std::cout << "Underrun" << std::endl;
    }
}

void SampleJumper::set_threshold_volume(const float decibels_0DBFS) {
    if (decibels_0DBFS > 0)
        throw std::invalid_argument("The decibel value must be less than or equal to zero.");
    
    this->set_max_sample_jump((int32_t) (sample_0dBFS * pow(10, (decibels_0DBFS/20))));
}

void SampleJumper::set_max_sample_jump(const int32_t  sample_val_threshold) {
    this->sample_diff_threshold = sample_val_threshold;
}

INDEX SampleJumper::try_compress(const INDEX& num_samples) {
    if (buffer.get_data_amount() == 0)
        return 0;
    
    INDEX samples_jumped = 0;
    INDEX diff;
    const INDEX size = buffer.get_data_amount();
    const INDEX last_index = size - 1;
    
    buffer.get_data(temp_buffer_1);
    
    for (int i = 0; i < size - 1; i++) {
        diff = abs(temp_buffer_1[i] - temp_buffer_1[i+1]);
        bool samples_left = num_samples - samples_jumped > 0;

        if (diff < sample_diff_threshold && samples_left) {
            temp_buffer_1[i - samples_jumped] = temp_buffer_1[i];
            samples_jumped++;
        }

        temp_buffer_1[i - samples_jumped] = temp_buffer_1[i];
    }

    // Last iteration is performed manually
    temp_buffer_1[last_index - samples_jumped] = temp_buffer_1[last_index];

    buffer.set_data(temp_buffer_1, size - samples_jumped);
    return samples_jumped;
}

INDEX SampleJumper::try_lengthen(const INDEX& num_samples) {
    if (buffer.get_data_amount() == 0)
        return 0;
    
    INDEX samples_jumped = 0;
    INDEX diff;
    const INDEX size = buffer.get_data_amount();
    const INDEX last_index = size - 1;
    
    buffer.get_data(temp_buffer_1);
    
    for (int i = 0; i < size - 1; i++) {
        diff = abs(temp_buffer_1[i] - temp_buffer_1[i + 1]);  
        bool samples_left = num_samples - samples_jumped > 0;
        
        if (diff < sample_diff_threshold && samples_left) {
            temp_buffer_2[i + samples_jumped] = temp_buffer_1[i];
            samples_jumped++;
        }
        
        temp_buffer_2[i + samples_jumped] = temp_buffer_1[i];
    }

    // Last iteration is performed manually
    temp_buffer_2[last_index + samples_jumped] = temp_buffer_1[last_index];

    buffer.set_data(temp_buffer_2, buffer.get_data_amount() + samples_jumped);
    return samples_jumped;
}

#undef SAMPLE
#undef INDEX