
#include <stdexcept>
#include <algorithm>
#include <math.h>

#include "TimeCompressor.h"

#ifndef SAMPLE
#define SAMPLE int32_t
#endif
#ifndef INDEX
#define INDEX int
#endif

const SAMPLE TimeCompressor::SAMPLE_MAX = (std::numeric_limits<SAMPLE>::max)();

TimeCompressor::TimeCompressor(const INDEX& expected_buffersize) {
    analysis_buffer = new std::pair<INDEX, SAMPLE>[expected_buffersize];
    analysis_buffer_size = expected_buffersize;
}

void TimeCompressor::set_volume_threshold(const float  decibels_0DBFS) {
    if (decibels_0DBFS > 0)
        throw std::invalid_argument("The decibel value must be less than or equal to zero.");
    
    this->set_value_threshold((SAMPLE) (SAMPLE_MAX * pow(10, (decibels_0DBFS/20))));
}

void TimeCompressor::set_value_threshold(const SAMPLE sample_val_threshold) {
    this->value_threshold = sample_val_threshold;
}

void TimeCompressor::set_action_interval(INDEX num_samples) {
    this->action_interval = num_samples;
}

void TimeCompressor::set_action_interval(std::chrono::microseconds us, INDEX samplerate) {
    this->set_action_interval(us.count() * samplerate / 1000000);
}

void TimeCompressor::set_target(INDEX target) {
    this->target = target;
}

void TimeCompressor::add_to_target(INDEX target_addition) {
    target += target_addition;
}



INDEX decide_which_samples_to_remove() {
    return 0;
}

INDEX TimeCompressor::remove_elements(SAMPLE* arr, std::pair<INDEX, SAMPLE>* indices, const size_t& size_arr, const size_t& size_indices) {
    INDEX elements_removed = 0;

    for (; elements_removed < size_indices - 1; elements_removed++) {
        SAMPLE* src = &arr[indices[elements_removed].first + 1];
        SAMPLE* dst = &arr[indices[elements_removed].first - elements_removed];
        INDEX size  = (indices[elements_removed + 1].first - indices[elements_removed].first) * sizeof(SAMPLE);
        memcpy(dst, src, size);
    }

    memcpy(
        &arr[indices[elements_removed].first - elements_removed],
        &arr[indices[elements_removed].first + 1],
        (size_arr - indices[size_indices-1].first) * sizeof(SAMPLE));

    return elements_removed + 1;
}

INDEX TimeCompressor::compressNext(SAMPLE* data, INDEX size) {
    //memcpy(data, source, size * sizeof(int32_t));

    if (target == 0)
        return 0;
    if (size < 3)
        throw std::invalid_argument("This algorithm requires at least 3 samples");
    
    if (size > analysis_buffer_size) {
        delete(analysis_buffer);
        analysis_buffer = new std::pair<INDEX, SAMPLE>[size];
        analysis_buffer_size = size;
    }

    INDEX samples_found = 0;

    /*
    First, we search for three samples that build a mostly flat curve.
    We use the sum of the difference between the two samples in each adjacent pair in the triplet, and check whether that sum is less than our threshold value.
    If it is, then we store the middle sample, essentially queing it for removal.
    */
    for (int i = 1; i < size - 1 && samples_found < target; i++) {
        SAMPLE totalJump = 
            abs(data[i-1] - data[i]) + // Adjacent pair 1
            abs(data[i] - data[i+1]);  // Adjacent pair 2

        if (totalJump < value_threshold) { // If the sum is less than our threshold value
            // Store the sample
            analysis_buffer[samples_found].first = i;
            analysis_buffer[samples_found].second = totalJump;
            samples_found++; // Keep track of how many samples we have found.
        }
    }

    if (samples_found == 0)
        return 0;
    
    // Sort the candidates based on their distance value.
    //std::sort(analysis_buffer, &analysis_buffer[samples_found - 1], [](const SampleEntry &sample_1, const SampleEntry &sample_2) {
    //    return sample_1.second < sample_2.second;
    //});

    // Decide which samples to cut.

    // Sort the candidates based on their index value.
    //std::sort(analysis_buffer, &analysis_buffer[samples_found - 1], [](const SampleEntry &sample_1, const SampleEntry &sample_2) {
    //    return sample_1.first < sample_2.first;
    //});

    remove_elements(data, analysis_buffer, size, samples_found);
    target -= samples_found;
    return samples_found;
}

#undef SAMPLE
#undef INDEX