

#include <math.h>
#include "audiotools/Sine.h"

#ifndef PI
#define PI 3.14159265
#endif

Sine::Sine(const float& freq, const int& samplerate) : Sine(freq, samplerate, (std::numeric_limits<int32_t>::max)()) {

}

Sine::Sine(const float& freq, const int& sr, const int32_t& amplitude) : 
    data_size((float) sr / freq),
    samplerate(sr),
    frequency((float) sr / data_size) {
    
    data = new int32_t[data_size];

    for (int i = 0; i < data_size; i++)
        data[i] = amplitude * sin((PI * 2 * i) / data_size);
}

void Sine::get_data(int32_t* dst, const int sample_count, const int offset) const {
    int first_transfer = std::min(sample_count, data_size - offset);

    memcpy(dst, &data[offset], first_transfer * sizeof(int32_t));
    if (sample_count == first_transfer)
        return;

    

    int index = first_transfer;

    for (; index < sample_count - data_size; index += data_size)
        memcpy(&dst[index], data, data_size * sizeof(int32_t));
    
    memcpy(&dst[index], data, (sample_count - index) * sizeof(int32_t));
}