#include "example/Basic_ASIO_Device.h"



BasicASIODevice::BasicASIODevice() {
	registry_index = init(nullptr);
}

BasicASIODevice::BasicASIODevice(AudioHost* listener) {
	registry_index = init(listener);
}

BasicASIODevice::~BasicASIODevice() {
	if (registry_index < 0)
		return;
	ASIO_callback_registry::removeEntry(registry_index);
}

int BasicASIODevice::init(AudioHost* listener) {
	int index = register_callbacks();
	set_listener(listener);
	return index;
}

int BasicASIODevice::register_callbacks() {
	int index = ASIO_callback_registry::registerEntry(this);
	if (index < 0)
		return index;

	ASIO_callback_registry::getCallbackPointers(index, &callbacks);

	return index;
}

void BasicASIODevice::set_listener(AudioHost* listener) {
	this->listener = listener;
}

// Inherited methods from 'ASIO_callback_registry_entry'. Only forwards the call to the member AudioHost 'listener'.
void      BasicASIODevice::bufferSwitch(long doubleBufferIndex, ASIOBool directProcess) { listener->bufferSwitch(doubleBufferIndex, directProcess, this); }
void      BasicASIODevice::sampleRateDidChange(ASIOSampleRate sRate) { listener->sampleRateDidChange(sRate, this); }
long      BasicASIODevice::asioMessage(long selector, long value, void* message, double* opt) { return listener->asioMessage(selector, value, message, opt, this); }
ASIOTime* BasicASIODevice::bufferSwitchTimeInfo(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess) { return listener->bufferSwitchTimeInfo(params, doubleBufferIndex, directProcess, this); }

void AudioHost_default::bufferSwitch(long doubleBufferIndex, ASIOBool directProcess, BasicASIODevice* caller) {

}

void AudioHost_default::sampleRateDidChange(ASIOSampleRate sRate, BasicASIODevice* caller) {

}

long AudioHost_default::asioMessage(long selector, long value, void* message, double* opt, BasicASIODevice* caller) {
	return 0L;
}

ASIOTime* AudioHost_default::bufferSwitchTimeInfo(ASIOTime* params, long doubleBufferIndex, ASIOBool directProcess, BasicASIODevice* caller) {
	return nullptr;
}
