
#include <iostream>
#include <string>
#include <vector>
#include <math.h>
#include <mutex>

#include "Instantiation_and_registry.h"
#include "ASIO_device.h"
#include "asio_sdk/ginclude.h"
#include "audiotools/Sine.h"

using namespace std;

class AudioEngine : public AudioHost {
private:
	long samplePos = 0L;
    Sine* s = nullptr;

public:

	void bufferSwitch(long doubleBufferIndex, bool directProcess, ASIODevice* caller) override {
		int32_t* bufferL = caller->get_output(0);
		int32_t* bufferR = caller->get_output(1);

        if (s == nullptr)
            s = new Sine(220, caller->get_sample_rate());
        
        s->get_data(bufferL, caller->get_buffer_size(), samplePos % s->get_data_size());
		memcpy(bufferR, bufferL, caller->get_buffer_size() * sizeof(int32_t));

		samplePos += caller->get_buffer_size();
	}

	long asioMessage(long selector, long value, void* message, double* opt, ASIODevice* caller) override {
		//cout << "#Selector1: " << selector << "   #Value: " << value << "   #Message: " << endl;

		char* message_char = (char*)message;

		// Device asking for supported selectors
		if (selector == kAsioSelectorSupported)
			switch (value) {
			case kAsioEngineVersion:	return 1L;
			case kAsioSupportsTimeInfo: return 1L;
			case kAsioResetRequest:		return 1L;
			case kAsioBufferSizeChange: return 1L;
			case kAsioResyncRequest:	return 0L;
			case kAsioLatenciesChanged: return 0L;
			case kAsioOverload:			return 0L;

			default:					return 0;
			}

		switch (selector) {
		case kAsioEngineVersion:	return 2L;
		case kAsioResetRequest:		return 0L;
		case kAsioResyncRequest:	return 0L;
		case kAsioSupportsTimeInfo: return 0L;
		case kAsioOverload:			return 0L;
		}

		return 0L;
	}

	ASIOTime* bufferSwitchTimeInfo(ASIOTime* params, long doubleBufferIndex, bool directProcess, ASIODevice* caller) override {
		return nullptr;
	}
};

int ask_for_device(string query) {
    const int num_devices = ASIO_SDK::get_num_ASIO_drivers();
    wstring* names = new wstring[num_devices];
    ASIO_SDK::read_driver_name_all(names);

    int index = 0;
    wstring* name = &names[0];
    while (index++ - num_devices)
        wcout << index << ": " << *name++ << endl;
    cout << index << ": [EXIT]" << endl;
	cout << query << endl;

	string s;
	cin >> s;
	long ans = strtol(s.c_str(), nullptr, 0);
	if (ans <= 0 || ans > num_devices + 1) {
		cout << "Did not understand the input." << endl;
		return -1;
	}

	if (ans == index)
		return -2;

    return (int) ans - 1;
}

int main() {
    int device_index_1 = ask_for_device("Choose a device to play from");
	AudioEngine engine_1;
    ASIODevice device_L(device_index_1, &engine_1);

    cout << "All good? [y/n]";
    string s;
    cin >> s;

	device_L.detach();
	
    
    return s == "y";
}